
breed [researchers researcher]
breed [journals journal]

globals [bandit-payoffs average-payoff-list random-guy] 

researchers-own [bandit-pulled experimental-result alphas betas learned-results]
journals-own[researchers-reported]

to setup
  clear-all
  
  ;; set up world
   set-default-shape turtles "circle"
   create-researchers initial-researchers
   layout-circle researchers (world-width / 2 - 1)
   create-journals initial-journals 
   layout-circle journals (world-width / 4 - 1)
   ask journals [ set shape "house" ] 

  ;; set up bandits
  set bandit-payoffs n-values number-of-bandits [  max  list (max-bandit-payoff - ( bandit-differences * ? )) 0 ]
  
  ;; set up beliefs
   ask researchers [
     let x (random initial-journals) + 1
     create-links-with n-of x journals
     set alphas n-values number-of-bandits [ random-float max-initial-alpha ]
     set betas n-values number-of-bandits [ random-float max-initial-alpha ]
   ]
   
   ;; set up globals
   set average-payoff-list []
   set random-guy n-of  number-of-publications researchers
end

to go
  ask-concurrent researchers [ experiment ]
  ask-concurrent journals [ publish ]
  ask-concurrent researchers [ update ]
  tick
  set average-payoff-list lput (mean [experimental-result] of researchers)  average-payoff-list
  do-plots
end 

to do-plots
  plot mean average-payoff-list
end

to experiment
  set experimental-result -1
  set bandit-pulled -1
  
  let expectations n-values number-of-bandits [ (item ? alphas) / (item ? alphas + item ? betas) ]
  
  foreach n-values number-of-bandits [?] [
    if item ? expectations = max expectations [
      set bandit-pulled ?
      repeat number-of-experiments [ 
        if random-float 1 < item ? bandit-payoffs [
          set experimental-result (experimental-result + 1)
        ]
      ]
    ]
  ]
  
  set color ((10 * bandit-pulled) + 5)
end

to publish
  set researchers-reported []
  
  if publishing-strategy = "best outcomes" [
    publish-best
  ]
  if publishing-strategy = "best and worst outcomes" [
    publish-best-worst
  ]
  if publishing-strategy = "worst outcomes" [
    publish-worst
  ]
  if publishing-strategy = "best from each bandit" [
    publish-best-each
  ]
  if publishing-strategy = "random each round" [
    publish-random-each-round
  ]
  if publishing-strategy = "random guy" [
    publish-random-guy
  ]
end

to publish-best
  ask link-neighbors [set shape "circle"]
  let researcher-neighbors sort-by [[experimental-result] of ?1 < [experimental-result] of ?2] link-neighbors    
  set researchers-reported []
  foreach n-values number-of-publications [?] [
    set researchers-reported (turtle-set  researchers-reported item ? researcher-neighbors)
  ]
  ask researchers-reported [set shape "square"]
end

to publish-best-worst
  ask link-neighbors [set shape "circle"]
  let researcher-neighbors sort-by [[experimental-result] of ?1 < [experimental-result] of ?2] link-neighbors    
  foreach n-values (number-of-publications / 2) [?] [
    set researchers-reported (turtle-set  researchers-reported item ? researcher-neighbors)
  ]
  foreach n-values (number-of-publications / 2) [?] [
    set researchers-reported (turtle-set researchers-reported item (length researcher-neighbors - ? - 1) researcher-neighbors)
  ]
  ask researchers-reported [set shape "square"]
end

to publish-worst
  ask link-neighbors [set shape "circle"]
  let researcher-neighbors reverse sort-by [[experimental-result] of ?1 < [experimental-result] of ?2] link-neighbors    
  set researchers-reported turtle-set []
  foreach n-values number-of-publications [?] [
    set researchers-reported (turtle-set  researchers-reported item ? researcher-neighbors)
  ]
  ask researchers-reported [set shape "square"]
end

to publish-best-each
   ask link-neighbors [set shape "circle"]
   set researchers-reported turtle-set []
   foreach n-values number-of-bandits [?] [
     let bandit ?
     let bandit-neighbors (link-neighbors with [bandit-pulled = bandit])
     set researchers-reported (turtle-set researchers-reported (bandit-neighbors with-max [experimental-result]))  
  ]
  ask researchers-reported [set shape "square"]
end

to publish-random-each-round
   ask link-neighbors [set shape "circle" ]
   set researchers-reported n-of number-of-publications researchers
   ask researchers-reported [set shape "square" ]
end

to publish-random-guy
  ask link-neighbors [set shape "circle" ]
   set researchers-reported random-guy
   ask researchers-reported [set shape "square" ]
end

to update
  ;; set a list of agents to update on
  set learned-results (turtle-set self ([researchers-reported] of link-neighbors))
  
  foreach sort learned-results [
    ;; this sucky bit of code, updates the agent's beliefs by each of the results in learned results
    set alphas replace-item ([bandit-pulled] of ?) alphas ( (item ([bandit-pulled] of ?) alphas) + ([experimental-result] of ?)) 
    set betas replace-item ([bandit-pulled] of ?) betas ( (item ([bandit-pulled] of ?) betas) + (number-of-experiments - ([experimental-result] of ?))) 
  ]
end  

@#$#@#$#@
GRAPHICS-WINDOW
222
10
609
418
14
14
13.0
1
10
1
1
1
0
0
0
1
-14
14
-14
14
0
0
1
ticks

BUTTON
16
36
83
69
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
15
87
187
120
initial-researchers
initial-researchers
1
50
10
1
1
NIL
HORIZONTAL

SLIDER
15
257
187
290
initial-journals
initial-journals
1
10
1
1
1
NIL
HORIZONTAL

SLIDER
619
10
791
43
max-bandit-payoff
max-bandit-payoff
0
1
0.5
.1
1
NIL
HORIZONTAL

SLIDER
620
49
792
82
bandit-differences
bandit-differences
0
1
0.01
.01
1
NIL
HORIZONTAL

SLIDER
621
130
823
163
number-of-experiments
number-of-experiments
1
1000
1000
10
1
NIL
HORIZONTAL

BUTTON
92
36
155
69
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
15
130
187
163
max-initial-alpha
max-initial-alpha
1
100
4
1
1
NIL
HORIZONTAL

SLIDER
15
172
187
205
max-initial-beta
max-initial-beta
1
100
4
1
1
NIL
HORIZONTAL

CHOOSER
13
340
222
385
publishing-strategy
publishing-strategy
"best outcomes" "best and worst outcomes" "worst outcomes" "best from each bandit" "random each round" "random guy"
5

SLIDER
621
90
793
123
number-of-bandits
number-of-bandits
2
100
2
1
1
NIL
HORIZONTAL

PLOT
628
196
828
346
Average payoff per round
round
payoff
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

SLIDER
13
298
214
331
number-of-publications
number-of-publications
2
10
2
2
1
NIL
HORIZONTAL

@#$#@#$#@
WHAT IS IT?
-----------

This is the simulation model used for the paper "Optimal publishing strategies" in Episteme.  It provides a model of scientific practice with centralized journals publishing the scientific results.

The underlying model is an n-armed Bandit problem.  Each scientist receives i.i.d draws from a bandit and "submits" her results to the central journal which selects particular results to publish using some strategy.  Each individual scientist then updates their beliefs based on the outcome they received and the others published by the journal.  Then on the next round they choose the arm which currently appears superior.  


PARAMETERS
----------

"initial-researchers" - The number of different scientists

"max-initial-alpha", "max-initial-beta" - The scientists use Beta distributions for their beliefs.  The initial parameters for these distributions are drawn from a uniform distribution over [0,x] where x is the value for this parameter

"initial-journals" - This is the number of potential journals.  Each scientist chooses a number of journals to connect to (uniform [0,x], where x is the number of journals).  The scientist then connects to a random collection of journals based on the number chosen.  Every scientist reads the results from every journal connect to and also submits her results to every journal connected to.  

"publishing strategy" - This is the publishing strategy used by the journal

"max-bandit-payoff" - This the probability of getting a "win" from the best bandit.  The remaining bandits are one, two, three, etc. increments of "bandit-differences" below this.

"bandit-differences" - See above. 

"number-of-bandits" - This is the number of different options available to each scientist.

"number-of-experiments" -  Each individual "experiment" is n pulls of one bandit.  This parameter determines that number



@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="Different publication strategies - 50 researchers, 3 journals, 2 publicationsd" repetitions="1000" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10"/>
    <metric>mean [experimental-result] of researchers</metric>
    <enumeratedValueSet variable="number-of-publications">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="bandit-differences">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-journals">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-initial-beta">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-researchers">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="publishing-strategy">
      <value value="&quot;best outcomes&quot;"/>
      <value value="&quot;best and worst outcomes&quot;"/>
      <value value="&quot;worst outcomes&quot;"/>
      <value value="&quot;best from each bandit&quot;"/>
      <value value="&quot;random each round&quot;"/>
      <value value="&quot;random guy&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number-of-experiments">
      <value value="501"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-initial-alpha">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-bandit-payoff">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number-of-bandits">
      <value value="2"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="DisplayData" repetitions="500" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="1000"/>
    <metric>(-1 * mean [bandit-pulled] of researchers ) + 1</metric>
    <enumeratedValueSet variable="max-initial-beta">
      <value value="4"/>
    </enumeratedValueSet>
    <steppedValueSet variable="number-of-publications" first="2" step="2" last="10"/>
    <enumeratedValueSet variable="number-of-bandits">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-journals">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-researchers">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="publishing-strategy">
      <value value="&quot;best outcomes&quot;"/>
      <value value="&quot;best and worst outcomes&quot;"/>
      <value value="&quot;worst outcomes&quot;"/>
      <value value="&quot;best from each bandit&quot;"/>
      <value value="&quot;random each round&quot;"/>
      <value value="&quot;random guy&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-initial-alpha">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number-of-experiments">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-bandit-payoff">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="bandit-differences">
      <value value="0.01"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
